#include "readertophone.h"
#include "ui_readertophone.h"
#include "PCSC/winscard.h"
#include "stdio.h"

#define SCARD_CTL_CODE(code)    (0x42000000 +(code))
unsigned long Rtn, Protocol;
SCARDCONTEXT hPCSC;
QStringList ReaderStr;
SCARDHANDLE hPicc;
unsigned char RxApdu[300];
unsigned char TxApdu[300];
QByteArray Data;
QString Str;
unsigned long RxLen, TxLen;



ReadertoPhone::ReadertoPhone(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::ReadertoPhone)
{
    ui->setupUi(this);
    CheckforReaders();
}

ReadertoPhone::~ReadertoPhone()
{
    delete ui;
}

void ReadertoPhone::CheckforReaders()
{
    unsigned long RdrStrSize = 0;
    ReaderStr.clear();
    SCardReleaseContext(hPCSC);
    if ((Rtn = SCardEstablishContext(SCARD_SCOPE_USER, NULL, NULL, &hPCSC)) != SCARD_S_SUCCESS) {
        //printf("SCardEstablishContext failed [0x%08lX]\r\n", Rtn);
        QString s = QString::number(Rtn, 16);
        ui->rtb_DisplayLog->append("SCardEstablishContext failed 0x" + s);
        return;
    }
    //printf("SCardEstablishContext success !!\r\n");
    ui->rtb_DisplayLog->append("SCardEstablishContext success !!");
    SCardListReaders(hPCSC, NULL, NULL, &RdrStrSize);
    if (RdrStrSize > 0) {
        char *RdrStr, *NxtRdrStr;
        RdrStr = (char *) malloc(sizeof(char) * RdrStrSize);
        NxtRdrStr = RdrStr;
        SCardListReaders(hPCSC, NULL, NxtRdrStr, &RdrStrSize);
        while (strlen(NxtRdrStr)) {
            //printf(" %s found\r\n", NxtRdrStr);
            ReaderStr.append(NxtRdrStr);
            NxtRdrStr = &NxtRdrStr[strlen(NxtRdrStr)+1];
        }
        ui->cbReader->addItems(ReaderStr);
        free(RdrStr);
    } else {
        //printf("No reader found\r\n");
        ui->rtb_DisplayLog->append("No reader found !");
        ReaderStr.append("No reader found");
        ui->cbReader->addItems(ReaderStr);
    }
}

bool ReadertoPhone::transmit()
{
    int i;
    Data.clear();
    for (i = 0; i < (int)TxLen; i++) {
        //printf("%2X\t",TxApdu[i]);
        Data[i] = TxApdu[i];
    }
    //printf("\n");
    Str.clear();
    Str.append(Data.toHex());
    for (i = 2; i < Str.length(); i += 3) {
        Str.insert(i, " ");
    }
    ui->rtb_DisplayLog->append("TxData: " + Str);
    RxLen = 300;
    if ((Rtn = SCardControl(hPicc, SCARD_CTL_CODE(3500), TxApdu, TxLen,RxApdu, 300, &RxLen)) != SCARD_S_SUCCESS) {
        //printf("SCardControl failed 0x%08lX\r\n", Rtn);
        QString s = QString::number(Rtn, 16);
        ui->rtb_DisplayLog->append("SCardControl failed 0x" + s);
        SCardDisconnect(hPicc, SCARD_UNPOWER_CARD);
        return false;
    }
    Data.clear();
    for (i = 0; i < (int)RxLen; i++) {
        //printf("%2X\t",RxApdu[i]);
        Data[i] = RxApdu[i];
    }
    //printf("\n");
    Str.clear();
    Str.append(Data.toHex());
    for (i = 2; i < Str.length(); i += 3) {
        Str.insert(i, " ");
    }
    ui->rtb_DisplayLog->append("RxData: " + Str);
    return true;
}

void ReadertoPhone::on_brefresh_clicked()
{
    ui->setCmd->setEnabled(false);
    ui->bconnect->setEnabled(false);
    ui->bRecieve->setEnabled(false);
    ui->bmode->setEnabled(false);
    ui->timeCmd->setEnabled(false);
    ui->cbReader->clear();
    CheckforReaders();
}

void ReadertoPhone::on_bdisconnect_clicked()
{
    ui->setCmd->setEnabled(false);
    ui->bconnect->setEnabled(false);
    ui->bRecieve->setEnabled(false);
    ui->bmode->setEnabled(false);
    ui->timeCmd->setEnabled(false);

    if (ui->bdisconnect->text() == "Connect") {
        SCardDisconnect(hPicc, SCARD_UNPOWER_CARD);
        if ((Rtn = SCardConnect(hPCSC, ui->cbReader->currentText().toAscii(),SCARD_SHARE_DIRECT, SCARD_PROTOCOL_UNDEFINED, &hPicc, &Protocol)) != SCARD_S_SUCCESS) {
            //printf("SCardConnect failed 0x%08lX\r\n", Rtn);
            QString s = QString::number(Rtn, 16);
            ui->rtb_DisplayLog->append("SCardConnect failed 0x" + s);
            return;
        }
        ui->setCmd->setEnabled(true);
        ui->bconnect->setEnabled(true);
        ui->bRecieve->setEnabled(true);
        ui->bmode->setEnabled(true);
        ui->timeCmd->setEnabled(true);
        //printf("Connect success !! \n");
        ui->rtb_DisplayLog->append("Connect success !!");
        ui->bdisconnect->setText("Disconnect");
    } else {
        SCardDisconnect(hPicc, SCARD_UNPOWER_CARD);
        //printf("Disconnect success !! \n");
        ui->rtb_DisplayLog->append("Disconnect success !!");
        ui->bdisconnect->setText("Connect");
    }
}

void ReadertoPhone::on_ConvertCmd_clicked()
{
    QString SNEP_URI = "D1 02 ";
    if (ui->uriTxt->text() != "") {
        if (ui->uriTxt->text().length() <= 50) {
            if((ui->uriTxt->text().length() + 5) < 16)
                SNEP_URI += "0";
            SNEP_URI += QString::number(ui->uriTxt->text().length() + 5, 16).toUpper();
            SNEP_URI += " 53 70 D1 01 ";
            if((ui->uriTxt->text().length() + 1) < 16)
                SNEP_URI += "0";
            SNEP_URI += QString::number(ui->uriTxt->text().length() + 1, 16).toUpper();
            SNEP_URI += " 55 03 ";
            for (int i = 0; i < ui->uriTxt->text().length(); i++)
                SNEP_URI += QString::number(ui->uriTxt->text()[i].toAscii(), 16).toUpper() + " ";
            ui->tdata->setText(SNEP_URI);
        }
    }
}

void ReadertoPhone::on_setCmd_clicked()
{
    memset(TxApdu, 0, 300);
    memset(RxApdu, 0, 300);
    TxApdu[0] = 0xE0;
    TxApdu[1] = 0x00;
    TxApdu[2] = 0x00;
    TxApdu[3] = 0x50;
    TxApdu[4] = ui->tdata->text().replace(" ","").length() /2;
    TxApdu[5] = 0xD1;
    TxApdu[6] = 0x02;
    TxApdu[7] = ui->uriTxt->text().length() + 5;
    TxApdu[8] = 0x53;
    TxApdu[9] = 0x70;
    TxApdu[10] = 0xD1;
    TxApdu[11] = 0x01;
    TxApdu[12] = ui->uriTxt->text().length() + 1;
    TxApdu[13] = 0x55;
    TxApdu[14] = 0x03;
    QByteArray qbatmp = ui->uriTxt->text().toAscii();
    for (int i = 0; i < qbatmp.size(); i++) {
        TxApdu[i + 15] = qbatmp[i];
    }
    TxLen = qbatmp.size() + 15;

    if(transmit()) {
        ui->rtb_DisplayLog->append("SNEP Message Set success !");
    } else {
        ui->rtb_DisplayLog->append("SNEP Message Set failed !");
    }
    ui->rtb_DisplayLog->append("==================================");
    ui->rtb_DisplayLog->moveCursor(QTextCursor::End);

}

void ReadertoPhone::on_bconnect_clicked()
{
    memset(TxApdu, 0, 300);
    memset(RxApdu, 0, 300);
    TxApdu[0] = 0xE0;
    TxApdu[1] = 0x00;
    TxApdu[2] = 0x00;
    TxApdu[3] = 0x40;
    TxApdu[4] = 0x03;
    TxApdu[5] = 0x06;
    TxApdu[6] = 0x01;
    TxApdu[7] = 0x02;
    TxLen = 8;

    if (ui->sp1Radio->isChecked())
        TxApdu[7] = 0x01;
    else if (ui->sp2Radio->isChecked())
        TxApdu[7] = 0x02;
    else if (ui->sp3Radio->isChecked())
        TxApdu[7] = 0x03;
    if (ui->mode1Radio->isChecked())
        TxApdu[6] = 0x01;
    else if (ui->mode2Radio->isChecked())
        TxApdu[6] = 0x02;

    if(transmit()) {
        ui->rtb_DisplayLog->append("Switched to Initiator Mode success !");
    } else {
        ui->rtb_DisplayLog->append("Switched to Initiator Mode failed !");
    }
    ui->rtb_DisplayLog->append("==================================");
    ui->rtb_DisplayLog->moveCursor(QTextCursor::End);
}

void ReadertoPhone::on_timeCmd_clicked()
{
    memset(TxApdu, 0, 300);
    memset(RxApdu, 0, 300);
    TxApdu[0] = 0xE0;
    TxApdu[1] = 0x00;
    TxApdu[2] = 0x00;
    TxApdu[3] = 0x41;
    TxApdu[4] = 0x02;
    TxApdu[5] = 0x00;
    TxApdu[6] = 0x00;
    TxLen = 7;

    int temp = ui->TimeoutUpDown->value();
    temp = temp * 100;
    TxApdu[5] = (unsigned char)(temp >> 8);
    TxApdu[6] = (unsigned char)(temp);

    if(transmit()) {
        ui->rtb_DisplayLog->append("Set Timeout success !");
    } else {
        ui->rtb_DisplayLog->append("Set Timeout failed !");
    }
    ui->rtb_DisplayLog->append("==================================");
    ui->rtb_DisplayLog->moveCursor(QTextCursor::End);
}

void ReadertoPhone::on_bRecieve_clicked()
{
    memset(TxApdu, 0, 300);
    memset(RxApdu, 0, 300);
    TxApdu[0] = 0xE0;
    TxApdu[1] = 0x00;
    TxApdu[2] = 0x00;
    TxApdu[3] = 0x99;
    TxApdu[4] = 0xC0;
    TxLen = 5;

    if(transmit()) {
        ui->rtb_DisplayLog->append("SNEP Message Recieve success !");
    } else {
        ui->rtb_DisplayLog->append("SNEP Message Recieve failed !");
    }
    ui->rtb_DisplayLog->append("==================================");
    ui->rtb_DisplayLog->moveCursor(QTextCursor::End);

    ui->tshowdata->clear();
    QString qshow;
    int i,index = 8;
    for (i = 0; i < RxApdu[6]; i++) {
        qshow  += QString(RxApdu[index++]);
    }
    ui->tshowdata->setText("Record Name: " + qshow);
    ui->tshowdata->append("Abbreviation: " + QString::number(RxApdu[index++], 16).toUpper());
    qshow.clear();
    for (i = 0; i < RxApdu[7] - 1; i++) {
        qshow  += QString(RxApdu[index++]);
    }
    ui->tshowdata->append(qshow);

}

void ReadertoPhone::on_bmode_clicked()
{
    memset(TxApdu, 0, 300);
    memset(RxApdu, 0, 300);
    TxApdu[0] = 0xE0;
    TxApdu[1] = 0x00;
    TxApdu[2] = 0x00;
    TxApdu[3] = 0x99;
    TxApdu[4] = 0x06;
    TxApdu[5] = 0x98;
    TxApdu[6] = 0x01;
    TxApdu[7] = 0x04;
    TxApdu[8] = 0x1A;
    TxApdu[9] = 0x01;
    TxApdu[10] = 0x6F;
    TxLen = 11;

    if(transmit()) {
        ui->rtb_DisplayLog->append("Switched to Target Mode success !");
    } else {
        ui->rtb_DisplayLog->append("Switched to Target Mode failed !");
    }
    ui->rtb_DisplayLog->append("==================================");
    ui->rtb_DisplayLog->moveCursor(QTextCursor::End);
}

void ReadertoPhone::on_bclear_clicked()
{
    ui->rtb_DisplayLog->clear();
}
